@@ -32,15 +32,16 @@
 ;;; Code:
 
 (defvar Info-default-directory-list
-  (let ((start (list "/usr/local/info/"
-		     "/usr/local/lib/info/"))
+  (let ((start (list "/usr/local/lib/info/"
+		     ;; This comes second so that, if it is the same
+		     ;; as configure-info-directory (which is usually true)
+		     ;; and Emacs has been installed (also usually true)
+		     ;; then the list will end with two copies of this;
+		     ;; which means that the last dir file Info-insert-dir
+		     ;; finds will be the one in this directory.
+		     "/usr/local/info/"))
 	(configdir (file-name-as-directory configure-info-directory)))
-    (or (member configdir start)
-	(setq start (nconc start (list configdir))))
-    (or (member (expand-file-name "../info/" data-directory) start)
-	(setq start
-	      (nconc start
-		     (list (expand-file-name "../info/" data-directory)))))
+    (setq start (nconc start (list configdir)))
     start)
   "List of directories to search for Info documentation files.
 They are searched in the order they are given in this list.
@@ -61,9 +62,7 @@ normally should come last (so that local files override standard ones).")
 (defvar gnus-default-nntp-server ""
   ;; set this to your local server
   "The name of the host running an NNTP server.
-If it is a string such as \":DIRECTORY\", then ~/DIRECTORY
-is used as a news spool.  `gnus-nntp-server' is initialised from NNTPSERVER
-environment variable or, if none, this value.")
+The null string means use the local host as the server site.")
 
 (defvar gnus-nntp-service "nntp"
   "NNTP service name, usually \"nntp\" or 119).
@@ -90,16 +89,17 @@ Will use `gnus-startup-file'-SERVER instead if exists.")
 (defconst rmail-spool-directory
   (cond ((string-match "^[^-]+-[^-]+-sco3.2v4" system-configuration)
 	 "/usr/spool/mail/")
-	;; SVR4 is said to use /var/mail, and make /usr/mail a link to that;
-	;; but since I'm not sure it's generally true, calling file-exists-p
-	;; prevents lossage if it isn't true. -- rms.
-	((and (string-match "^[^-]+-[^-]+-sysv4" system-configuration)
-	      (file-exists-p "/var/mail"))
+	;; On The Bull DPX/2 /usr/spool/mail is used although 
+	;; it is usg-unix-v.
+	((string-match "^m68k-bull-sysv3" system-configuration)
+	 "/usr/spool/mail/")
+	;; SVR4 and recent BSD are said to use this.
+	;; Rather than trying to know precisely which systems use it,
+	;; let's assume this dir is never used for anything else.
+	((file-exists-p "/var/mail")
 	 "/var/mail/")
 	((memq system-type '(dgux hpux usg-unix-v unisoft-unix rtu irix))
 	 "/usr/mail/")
-	((eq system-type 'netbsd)
-	 "/var/mail/")
 	(t "/usr/spool/mail/"))
   "Name of directory used by system mailer for delivering new mail.
 Its name should end with a slash.")
@@ -112,6 +112,26 @@ Its name should end with a slash.")
     (t "fakemail"))			;In ../etc, to interface to /bin/mail.
   "Program used to send messages.")
 
+(defconst remote-shell-program
+  (cond
+   ;; Some systems use rsh for the remote shell; others use that name for the
+   ;; restricted shell and use remsh for the remote shell.  Let's try to guess
+   ;; based on what we actually find out there.  The restricted shell is
+   ;; almost certainly in /bin or /usr/bin, so it's probably safe to assume
+   ;; that an rsh found elsewhere is the remote shell program.  The converse
+   ;; is not true: /usr/bin/rsh could be either one, so check that last.
+   ((file-exists-p "/usr/ucb/remsh") "/usr/ucb/remsh")
+   ((file-exists-p "/usr/bsd/remsh") "/usr/bsd/remsh")
+   ((file-exists-p "/bin/remsh") "/bin/remsh")
+   ((file-exists-p "/usr/bin/remsh") "/bin/remsh")
+   ((file-exists-p "/usr/local/bin/remsh") "/usr/local/bin/remsh")
+   ((file-exists-p "/usr/ucb/rsh") "/usr/ucb/rsh")
+   ((file-exists-p "/usr/bsd/rsh") "/usr/bsd/rsh")
+   ((file-exists-p "/usr/local/bin/rsh") "/usr/local/bin/rsh")
+   ((file-exists-p "/bin/rsh") "/bin/rsh")
+   ((file-exists-p "/usr/bin/rsh") "/usr/bin/rsh")
+   (t "rsh")))
+
 (defconst term-file-prefix (if (eq system-type 'vax-vms) "[.term]" "term/")
   "If non-nil, Emacs startup does (load (concat term-file-prefix (getenv \"TERM\")))
 You may set this variable to nil in your `.emacs' file if you do not wish
