@@ -37,17 +37,6 @@ NAME-EXP may be a string constant giving the file name to use,
 or more generally it may be any kind of expression that returns
 a file name as a string.")
 
-(defun rmail-output-menu (event)
-  "Output current message to another Rmail file, chosen with a menu.
-Also set the default for subsequent \\[rmail-output-to-rmail-file] commands.
-The variables `rmail-secondary-file-directory' and
-`rmail-secondary-file-regexp' control which files are offered in the menu."
-  (interactive "e")
-  (let ((file-name (rmail-secondary-file-menu event)))
-    (if file-name
-	(rmail-output-to-rmail-file
-	 (setq rmail-default-rmail-file file-name)))))
-
 ;;; There are functions elsewhere in Emacs that use this function; check
 ;;; them out before you change the calling method.
 (defun rmail-output-to-rmail-file (file-name &optional count)
@@ -130,7 +119,17 @@ starting with the current one.  Deleted messages are skipped and don't count."
 			(beg (1+ (rmail-msgbeg rmail-current-message)))
 			(end (1+ (rmail-msgend rmail-current-message))))
 		    (if (not buf)
-			(append-to-file beg end file-name)
+			;; Output to a file.
+			(if rmail-fields-not-to-output
+			    ;; Delete some fields while we output.
+			    (let ((obuf (current-buffer)))
+			      (set-buffer (get-buffer-create " rmail-out-temp"))
+			      (insert-buffer-substring obuf beg end)
+			      (rmail-delete-unwanted-fields)
+			      (append-to-file (point-min) (point-max) file-name)
+			      (set-buffer obuf)
+			      (kill-buffer (get-buffer " rmail-out-temp")))
+			  (append-to-file beg end file-name))
 		      (if (eq buf (current-buffer))
 			  (error "Can't output message to same file it's already in"))
 		      ;; File has been visited, in buffer BUF.
@@ -154,6 +153,7 @@ starting with the current one.  Deleted messages are skipped and don't count."
 			      (widen)
 			      (search-backward "\n\^_")
 			      (narrow-to-region (point) (point-max))
+			      (rmail-delete-unwanted-fields)
 			      (rmail-count-new-messages t)
 			      (if (rmail-summary-exists)
 				  (rmail-select-summary
@@ -162,7 +162,8 @@ starting with the current one.  Deleted messages are skipped and don't count."
 		;; Output file not in rmail mode => just insert at the end.
 		(narrow-to-region (point-min) (1+ (buffer-size)))
 		(goto-char (point-max))
-		(insert-buffer-substring cur beg end)))))))
+		(insert-buffer-substring cur beg end)
+		(rmail-delete-unwanted-fields)))))))
 	      (rmail-set-attribute "filed" t))
 	  (if redelete (rmail-set-attribute "deleted" t))))
       (setq count (1- count))
@@ -171,6 +172,26 @@ starting with the current one.  Deleted messages are skipped and don't count."
 	(if (> count 0)
 	    (rmail-next-undeleted-message 1))))))
 
+(defvar rmail-fields-not-to-output nil
+  "*Regexp describing fields to exclude when outputting a message to a file.")
+
+;; Delete from the buffer header fields we don't want output.
+;; NOT-RMAIL if t means this buffer does not have the full header
+;; and *** EOOH *** that a message in an Rmail file has.
+(defun rmail-delete-unwanted-fields (&optional not-rmail)
+  (if rmail-fields-not-to-output 
+      (save-excursion
+	(goto-char (point-min))
+	;; Find the end of the header.
+	(if (and (or not-rmail (search-forward "\n*** EOOH ***\n" nil t))
+		 (search-forward "\n\n" nil t))
+	    (let ((end (point-marker)))
+	      (goto-char (point-min))
+	      (while (re-search-forward rmail-fields-not-to-output end t)
+		(beginning-of-line)
+		(delete-region (point)
+			       (progn (forward-line 1) (point)))))))))
+
 ;; Returns t if file FILE is an Rmail file.
 ;;;###autoload
 (defun rmail-file-p (file)
@@ -265,6 +286,7 @@ The optional fourth argument FROM-GNUS is set when called from GNUS."
 	  (set-buffer tembuf)
 	  (erase-buffer)
 	  (insert-buffer-substring rmailbuf)
+	  (rmail-delete-unwanted-fields t)
 	  (insert "\n")
 	  (goto-char (point-min))
 	  (if mail-from
@@ -278,9 +300,10 @@ The optional fourth argument FROM-GNUS is set when called from GNUS."
 	  ;; ``Quote'' "\nFrom " as "\n>From "
 	  ;;  (note that this isn't really quoting, as there is no requirement
 	  ;;   that "\n[>]+From " be quoted in the same transparent way.)
-	  (while (search-forward "\nFrom " nil t)
-	    (forward-char -5)
-	    (insert ?>))
+	  (let ((case-fold-search nil))
+	    (while (search-forward "\nFrom " nil t)
+	      (forward-char -5)
+	      (insert ?>)))
 	  (write-region (point-min) (point-max) file-name t
 			(if noattribute 'nomsg)))
 	(or noattribute
