@@ -1,6 +1,6 @@
 ;;; solar.el --- calendar functions for solar events.
 
-;; Copyright (C) 1992 Free Software Foundation, Inc.
+;; Copyright (C) 1992, 1993 Free Software Foundation, Inc.
 
 ;; Author: Edward M. Reingold <reingold@cs.uiuc.edu>
 ;; Keywords: calendar
@@ -9,25 +9,24 @@
 
 ;; This file is part of GNU Emacs.
 
-;; GNU Emacs is distributed in the hope that it will be useful,
-;; but WITHOUT ANY WARRANTY.  No author or distributor
-;; accepts responsibility to anyone for the consequences of using it
-;; or for whether it serves any particular purpose or works at all,
-;; unless he says so in writing.  Refer to the GNU Emacs General Public
-;; License for full details.
+;; GNU Emacs is free software; you can redistribute it and/or modify
+;; it under the terms of the GNU General Public License as published by
+;; the Free Software Foundation; either version 2, or (at your option)
+;; any later version.
 
-;; Everyone is granted permission to copy, modify and redistribute
-;; GNU Emacs, but only under the conditions described in the
-;; GNU Emacs General Public License.   A copy of this license is
-;; supposed to have been given to you along with GNU Emacs so you
-;; can know your rights and responsibilities.  It should be in a
-;; file named COPYING.  Among other things, the copyright notice
-;; and this notice must be preserved on all copies.
+;; GNU Emacs is distributed in the hope that it will be useful,
+;; but WITHOUT ANY WARRANTY; without even the implied warranty of
+;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+;; GNU General Public License for more details.
+
+;; You should have received a copy of the GNU General Public License
+;; along with GNU Emacs; see the file COPYING.  If not, write to
+;; the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
 ;;; Commentary:
 
-;; This collection of functions implements the features of calendar.el and
-;; diary.el that deal with times of day, sunrise/sunset, and
+;; This collection of functions implements the features of calendar.el,
+;; diary.el, and holiday.el that deal with times of day, sunrise/sunset, and
 ;; eqinoxes/solstices.
 
 ;; Based on the ``Almanac for Computers 1984,'' prepared by the Nautical
@@ -55,7 +54,7 @@
 
 (if (fboundp 'atan)
     (require 'lisp-float-type)
-  (error "Solar calculations impossible since floating point is unavailable."))
+  (error "Solar/lunar calculations impossible since floating point is unavailable."))
 
 (require 'cal-dst)
 
@@ -78,32 +77,69 @@ would give military-style times like `21:07 (UTC)'.")
 
 ;;;###autoload
 (defvar calendar-latitude nil
-  "*Latitude of `calendar-location-name' in degrees, + north, - south.
-For example, 40.7 for New York City.
-It may not be a good idea to set this in advance for your site;
-if there may be users running Emacs at your site
-who are physically located elsewhere, they would get the wrong
-value and might not know how to override it.")
+  "*Latitude of `calendar-location-name' in degrees.
+
+The value can be either a decimal fraction (one place of accuracy is
+sufficient), + north, - south, such as 40.7 for New York City, or the value
+can be a vector [degrees minutes north/south] such as [40 50 north] for New
+York City.
+
+This variable should be set in site-local.el.")
 
 ;;;###autoload
 (defvar calendar-longitude nil
-  "*Longitude of `calendar-location-name' in degrees, + east, - west.
-For example, -74.0 for New York City.
-It may not be a good idea to set this in advance for your site;
-if there may be users running Emacs at your site
-who are physically located elsewhere, they would get the wrong
-value and might not know how to override it.")
+  "*Longitude of `calendar-location-name' in degrees.
+
+The value can be either a decimal fraction (one place of accuracy is
+sufficient), + east, - west, such as -73.9 for New York City, or the value
+can be a vector [degrees minutes east/west] such as [73 55 west] for New
+York City.
+
+This variable should be set in site-local.el.")
+
+(defsubst calendar-latitude ()
+  "Convert calendar-latitude to a signed decimal fraction, if needed."
+  (if (numberp calendar-latitude)
+      calendar-latitude
+    (let ((lat (+ (aref calendar-latitude 0)
+                  (/ (aref calendar-latitude 1) 60.0))))
+      (if (equal (aref calendar-latitude 2) 'north)
+          lat
+        (- lat)))))
+
+(defsubst calendar-longitude ()
+  "Convert calendar-longitude to a signed decimal fraction, if needed."
+  (if (numberp calendar-longitude)
+      calendar-longitude
+    (let ((long (+ (aref calendar-longitude 0)
+                  (/ (aref calendar-longitude 1) 60.0))))
+      (if (equal (aref calendar-longitude 2) 'east)
+          long
+        (- long)))))
 
 ;;;###autoload
 (defvar calendar-location-name
   '(let ((float-output-format "%.1f"))
      (format "%s%s, %s%s"
-	     (abs calendar-latitude)
-	     (if (> calendar-latitude 0) "N" "S")
-	     (abs calendar-longitude)
-	     (if (> calendar-longitude 0) "E" "W")))
+             (if (numberp calendar-latitude)
+                 (abs calendar-latitude)
+               (+ (aref calendar-latitude 0)
+                  (/ (aref calendar-latitude 1) 60.0)))
+             (if (numberp calendar-latitude)
+                 (if (> calendar-latitude 0) "N" "S")
+               (if (equal (aref calendar-latitude 2) 'north) "N" "S"))
+             (if (numberp calendar-longitude)
+                 (abs calendar-longitude)
+               (+ (aref calendar-longitude 0)
+                  (/ (aref calendar-longitude 1) 60.0)))
+             (if (numberp calendar-longitude)
+                 (if (> calendar-longitude 0) "E" "W")
+               (if (equal (aref calendar-longitude 2) 'east) "E" "W"))))
   "*Expression evaluating to name of `calendar-longitude', calendar-latitude'.
-Default value is just the latitude, longitude pair.")
+For example, \"New York City\".  Default value is just the latitude, longitude
+pair.
+
+This variable should be set in site-local.el.")
 
 (defvar solar-n-hemi-seasons
   '("Vernal Equinox" "Summer Solstice" "Autumnal Equinox" "Winter Solstice")
@@ -136,10 +172,10 @@ Returns nil if nothing was entered."
     (if (not (string-equal x ""))
         (string-to-int x))))
 
-(defun solar-sin-degrees (x)
+(defsubst solar-sin-degrees (x)
   (sin (degrees-to-radians x)))
 
-(defun solar-cosine-degrees (x)
+(defsubst solar-cosine-degrees (x)
   (cos (degrees-to-radians x)))
 
 (defun solar-tangent-degrees (x)
@@ -183,11 +219,11 @@ Returns nil if nothing was entered."
 (defconst solar-earth-orbit-eccentricity 0.016718
   "Eccentricity of orbit of the earth around the sun.")
 
-(defmacro solar-degrees-to-hours (deg)
-  (list '/ deg 15))
+(defsubst solar-degrees-to-hours (deg)
+  (/ deg 15.0))
 
-(defmacro solar-hours-to-days (hour)
-  (list '/ hour 24))
+(defsubst solar-hours-to-days (hour)
+  (/ hour 24.0))
 
 (defun solar-longitude-of-sun (day)
   "Longitude of the sun at DAY in the year."
@@ -212,14 +248,17 @@ Returns nil if nothing was entered."
       (solar-sin-degrees longitude))))
 
 (defun solar-sunrise (date)
-  "Calculates the *standard* time of sunrise for Gregorian DATE for location
-given by `calendar-latitude' and `calendar-longitude'.  Returns a decimal fraction
-of hours.  Returns nil if the sun does not rise at that location on that day."
+  "Calculates the *standard* time of sunrise for Gregorian DATE.
+Calculation is for location given by `calendar-latitude' and
+`calendar-longitude'.
+
+Returns a decimal fraction of hours.  Returns nil if the sun does not rise at
+that location on that day."
   (let* ((day-of-year (calendar-day-number date))
 	 (approx-sunrise
           (+ day-of-year
              (solar-hours-to-days
-              (-  6 (solar-degrees-to-hours calendar-longitude)))))
+              (-  6 (solar-degrees-to-hours (calendar-longitude))))))
 	 (solar-longitude-of-sun-at-sunrise
           (solar-longitude-of-sun approx-sunrise))
 	 (solar-right-ascension-at-sunrise
@@ -229,9 +268,9 @@ of hours.  Returns nil if the sun does not rise at that location on that day."
 	 (cos-local-sunrise
           (/ (- (solar-cosine-degrees (+ 90 (/ 50.0 60.0)))
                 (* (solar-sin-degrees solar-declination-at-sunrise)
-                   (solar-sin-degrees calendar-latitude)))
+                   (solar-sin-degrees (calendar-latitude))))
              (* (solar-cosine-degrees solar-declination-at-sunrise)
-                (solar-cosine-degrees calendar-latitude)))))
+                (solar-cosine-degrees (calendar-latitude))))))
     (if (<= (abs cos-local-sunrise) 1);; otherwise, no sunrise that day
       (let* ((local-sunrise (solar-degrees-to-hours
                              (- 360 (solar-arccos cos-local-sunrise))))
@@ -240,18 +279,21 @@ of hours.  Returns nil if the sun does not rise at that location on that day."
 		      (+ (* 0.065710 approx-sunrise)
 			 6.622))
 		   24)))
-	(+ (- local-mean-sunrise (solar-degrees-to-hours calendar-longitude))
+	(+ (- local-mean-sunrise (solar-degrees-to-hours (calendar-longitude)))
 	   (/ calendar-time-zone 60.0))))))
 
 (defun solar-sunset (date)
-  "Calculates the *standard* time of sunset for Gregorian DATE for location
-given by `calendar-latitude' and `calendar-longitude'.  Returns a decimal fractions
-of hours.  Returns nil if the sun does not set at that location on that day."
+  "Calculates the *standard* time of sunset for Gregorian DATE.
+Calculation is for location given by `calendar-latitude' and
+`calendar-longitude'.
+
+Returns a decimal fractions of hours.  Returns nil if the sun does not set at
+that location on that day."
   (let* ((day-of-year (calendar-day-number date))
 	 (approx-sunset
           (+ day-of-year
              (solar-hours-to-days
-              (- 18 (solar-degrees-to-hours calendar-longitude)))))
+              (- 18 (solar-degrees-to-hours (calendar-longitude))))))
 	 (solar-longitude-of-sun-at-sunset
           (solar-longitude-of-sun approx-sunset))
 	 (solar-right-ascension-at-sunset
@@ -261,9 +303,9 @@ of hours.  Returns nil if the sun does not set at that location on that day."
 	 (cos-local-sunset
           (/ (- (solar-cosine-degrees (+ 90 (/ 50.0 60.0)))
                 (* (solar-sin-degrees solar-declination-at-sunset)
-                   (solar-sin-degrees calendar-latitude)))
+                   (solar-sin-degrees (calendar-latitude))))
              (* (solar-cosine-degrees solar-declination-at-sunset)
-                (solar-cosine-degrees calendar-latitude)))))
+                (solar-cosine-degrees (calendar-latitude))))))
     (if (<= (abs cos-local-sunset) 1);; otherwise, no sunset that day
       (let* ((local-sunset (solar-degrees-to-hours
                             (solar-arccos cos-local-sunset)))
@@ -271,23 +313,28 @@ of hours.  Returns nil if the sun does not set at that location on that day."
 	      (mod (- (+ local-sunset solar-right-ascension-at-sunset)
 		      (+ (* 0.065710 approx-sunset) 6.622))
 		   24)))
-	(+ (- local-mean-sunset (solar-degrees-to-hours calendar-longitude))
+	(+ (- local-mean-sunset (solar-degrees-to-hours (calendar-longitude)))
 	   (/ calendar-time-zone 60.0))))))
 
-(defun solar-time-string (time date &optional style)
-  "Printable form for decimal fraction *standard* TIME on DATE.
-Optional parameter STYLE forces the time to be standard time when its value
-is 'standard and daylight savings time (if available) when its value is
+(defun solar-adj-time-for-dst (date time &optional style)
+  "Adjust decimal fraction standard TIME on DATE to account for dst.
+Returns a list (date adj-time zone) where `date' and `time' are the values
+adjusted for `zone'; here `date' is a list (month day year), `time' is a
+decimal fraction time, and `zone' is a string.
+
+Optional parameter STYLE forces the result time to be standard time when its
+value is 'standard and daylight savings time (if available) when its value is
 'daylight.
 
-Format used is given by `calendar-time-display-form'.  Converted to daylight
-savings time according to `calendar-daylight-savings-starts',
-`calendar-daylight-savings-ends', `calendar-daylight-savings-starts-time',
-`calendar-daylight-savings-ends-time', and `calendar-daylight-savings-offset'."
+Conversion to daylight savings time is done according to
+`calendar-daylight-savings-starts', `calendar-daylight-savings-ends',
+`calendar-daylight-savings-starts-time',
+`calendar-daylight-savings-ends-time', and
+`calendar-daylight-savings-offset'."
+
   (let* ((year (extract-calendar-year date))
-	 (time (round (* 60 time)))
 	 (rounded-abs-date (+ (calendar-absolute-from-gregorian date)
-			      (/ time 60.0 24.0)))
+			      (/ (round (* 60 time)) 60.0 24.0)))
          (dst-starts (and calendar-daylight-savings-starts
                           (+ (calendar-absolute-from-gregorian
                               (eval calendar-daylight-savings-starts))
@@ -315,7 +362,16 @@ savings time according to `calendar-daylight-savings-starts',
 	 (time-zone (if dst
 			calendar-daylight-time-zone-name
 			calendar-standard-time-zone-name))
-	 (time (+ time (if dst calendar-daylight-time-offset 0)))
+	 (time (+ rounded-abs-date
+                  (if dst (/ calendar-daylight-time-offset 24.0 60.0) 0))))
+    (list (calendar-gregorian-from-absolute (truncate time))
+          (* 24.0 (- time (truncate time)))
+          time-zone)))
+
+(defun solar-time-string (time time-zone)
+  "Printable form for decimal fraction TIME on DATE.
+Format used is given by `calendar-time-display-form'."
+  (let* ((time (round (* 60 time)))
 	 (24-hours (/ time 60))
 	 (minutes (format "%02d" (% time 60)))
 	 (12-hours (format "%d" (1+ (% (+ 24-hours 11) 12))))
@@ -325,14 +381,16 @@ savings time according to `calendar-daylight-savings-starts',
 
 (defun solar-sunrise-sunset (date)
   "String giving local times of sunrise and sunset on Gregorian DATE."
-  (let ((rise (solar-sunrise date))
-	(set (solar-sunset date)))
+  (let* ((rise (solar-sunrise date))
+         (adj-rise (if rise (solar-adj-time-for-dst date rise)))
+         (set (solar-sunset date))
+         (adj-set (if set (solar-adj-time-for-dst date set))))
     (format "%s, %s at %s"
-	    (if rise
-		(concat "Sunrise " (solar-time-string rise date))
+	    (if (and rise (calendar-date-equal date (car adj-rise)))
+		(concat "Sunrise " (apply 'solar-time-string (cdr adj-rise)))
 	      "No sunrise")
-	    (if set
-		(concat "sunset " (solar-time-string set date))
+	    (if (and set (calendar-date-equal date (car adj-set)))
+		(concat "sunset " (apply 'solar-time-string (cdr adj-set)))
 	      "no sunset")
 	    (eval calendar-location-name))))
 
@@ -393,13 +451,14 @@ several minutes."
 ;;;###autoload
 (defun sunrise-sunset (&optional arg)
   "Local time of sunrise and sunset for today.  Accurate to +/- 2 minutes.
-If called with an optional prefix argument, prompts for date.
+If called with an optional prefix argument, prompt for date.
 
-If called with an optional double prefix argument, prompts for longitude,
-latitude, time zone, and date.
+If called with an optional double prefix argument, prompt for longitude,
+latitude, time zone, and date, and always use standard time.
 
 This function is suitable for execution in a .emacs file."
  (interactive "p")
+ (or arg (setq arg 1))
  (if (and (< arg 16)
           (not (and calendar-latitude calendar-longitude calendar-time-zone)))
      (solar-setup))
@@ -419,10 +478,21 @@ This function is suitable for execution in a .emacs file."
          (if (< arg 16) calendar-location-name
            (let ((float-output-format "%.1f"))
              (format "%s%s, %s%s"
-                     (abs calendar-latitude)
-                     (if (> calendar-latitude 0) "N" "S")
-                     (abs calendar-longitude)
-                     (if (> calendar-longitude 0) "E" "W")))))
+                     (if (numberp calendar-latitude)
+                         (abs calendar-latitude)
+                       (+ (aref calendar-latitude 0)
+                          (/ (aref calendar-latitude 1) 60.0)))
+                     (if (numberp calendar-latitude)
+                         (if (> calendar-latitude 0) "N" "S")
+                       (if (equal (aref calendar-latitude 2) 'north) "N" "S"))
+                     (if (numberp calendar-longitude)
+                         (abs calendar-longitude)
+                       (+ (aref calendar-longitude 0)
+                          (/ (aref calendar-longitude 1) 60.0)))
+                     (if (numberp calendar-longitude)
+                         (if (> calendar-longitude 0) "E" "W")
+                       (if (equal (aref calendar-longitude 2) 'east)
+                           "E" "W"))))))
         (calendar-standard-time-zone-name
          (if (< arg 16) calendar-standard-time-zone-name
            (cond ((= calendar-time-zone 0) "UTC")
@@ -455,10 +525,10 @@ Accurate to +/- 2 minutes."
   (interactive)
   (if (not (and calendar-latitude calendar-longitude calendar-time-zone))
       (solar-setup))
-  (message
-   (solar-sunrise-sunset
-    (or (calendar-cursor-to-date)
-	(error "Cursor is not on a date!")))))
+  (let ((date (calendar-cursor-to-date t)))
+    (message "%s: %s"
+             (calendar-date-string date t t)
+             (solar-sunrise-sunset date))))
 
 (defun diary-sunrise-sunset ()
   "Local time of sunrise and sunset as a diary entry.
@@ -474,10 +544,15 @@ No diary entry if there is no sunset on that date."
       (solar-setup))
   (if (= (% (calendar-absolute-from-gregorian date) 7) 5);;  Friday
       (let* ((sunset (solar-sunset date))
-	     (light (if sunset (- sunset (/ 18.0 60.0)))))
-        (if light (format "%s Sabbath candle lighting"
-                          (solar-time-string light date))))))
+	     (light (if sunset
+                        (solar-adj-time-for-dst
+                         date
+                         (- sunset (/ 18.0 60.0))))))
+        (if (and light (calendar-date-equal date (car light)))
+            (format "%s Sabbath candle lighting"
+                    (apply 'solar-time-string (cdr light)))))))
 
+;;;###autoload
 (defun solar-equinoxes-solstices ()
   "Date and time of equinoxes and solstices, if visible in the calendar window.
 Requires floating point."
@@ -495,19 +570,18 @@ Requires floating point."
            (calendar-time-zone (if calendar-time-zone calendar-time-zone 0))
            (k (1- (/ m 3)))
 	   (date (solar-equinoxes/solstices k y))
+	   (s-hemi (and calendar-latitude (< (calendar-latitude) 0)))
 	   (day (extract-calendar-day date))
-	   (time (* 24 (- day (truncate day))))
-	   (s-hemi (and calendar-latitude (< calendar-latitude 0)))
-           ;; Time zone/DST can't move the date out of range,
-           ;; so let solar-time-string do the conversion.
-	   (date (list (extract-calendar-month date)
+           (adj (solar-adj-time-for-dst
+                 (list (extract-calendar-month date)
 		       (truncate day)
-		       (extract-calendar-year date))))
-      (list (list date
+		       (extract-calendar-year date))
+                 (* 24 (- day (truncate day))))))
+      (list (list (car adj)
 		  (format "%s %s"
 			  (nth k (if s-hemi solar-s-hemi-seasons
                                    solar-n-hemi-seasons))
-			  (solar-time-string time date)))))))
+			  (apply 'solar-time-string (cdr adj))))))))
 
 (provide 'solar)
 
