@@ -1,8 +1,9 @@
 ;;; yow.el --- quote random zippyisms
 
-;; Copyright (C) 1993 Free Software Foundation, Inc.
+;; Copyright (C) 1993, 1994 Free Software Foundation, Inc.
 
 ;; Maintainer: FSF
+;; Author: Richard Mlynarik
 ;; Keywords: games
 
 ;; This file is part of GNU Emacs.
@@ -23,26 +24,28 @@
 
 ;;; Commentary:
 
-;; Important pinheaddery for GNU Emacs.
+;; Important pinheadery for GNU Emacs.
 ;;
-;; See cookie.el for implementation.  Note --- the `n' argument of yow
+;; See cookie1.el for implementation.  Note --- the `n' argument of yow
 ;; from the 18.xx implementation is no longer; we only support *random*
 ;; random access now.
 
 ;;; Code:
 
-(require 'cookie)
+(require 'cookie1)
 
 (defvar yow-file (concat data-directory "yow.lines")
-   "Pertinent pinhead phrases.")
+   "File containing pertinent pinhead phrases.")
 
 ;;;###autoload
-(defun yow (&optional interactive)
-  "Return or display a random Zippy quotation."
-  (interactive "p")
-  (let ((yow (cookie
-	      yow-file "Am I CONSING yet?..." "I have SEEN the CONSING!!")))
-    (cond ((not interactive)
+(defun yow (&optional insert)
+  "Return or display a random Zippy quotation.  With prefix arg, insert it."
+  (interactive "P")
+  (let ((yow (cookie yow-file
+		     "Am I CONSING yet?..." "I have SEEN the CONSING!!")))
+    (cond (insert
+	   (insert yow))
+	  ((not (interactive-p))
 	   yow)
 	  ((not (string-match "\n" yow))
 	   (delete-windows-on (get-buffer-create "*Help*"))
@@ -50,8 +53,23 @@
 	  (t
 	   (message "Yow!")
 	   (with-output-to-temp-buffer "*Help*"
-	     (princ yow))))))
+	     (princ yow)
+	     (save-excursion
+	       (set-buffer standard-output)
+	       (help-mode)))))))
 
+(defsubst read-zippyism (prompt &optional require-match)
+  "Read a Zippyism from the minibuffer with completion, prompting with PROMPT.
+If optional second arg is non-nil, require input to match a completion."
+  (read-cookie prompt yow-file
+	       "Am I CONSING yet?..." "I have SEEN the CONSING!!"
+	       require-match))
+
+;;;###autoload
+(defun insert-zippyism (&optional zippyism)
+  "Prompt with completion for a known Zippy quotation, and insert it at point."
+  (interactive (list (read-zippyism "Pinhead wisdom: " t)))
+  (insert zippyism))
 
 ; Yowza!! Feed zippy quotes to the doctor. Watch results.
 ; fun, fun, fun. Entertainment for hours...
