@@ -5,7 +5,7 @@ This file is part of GNU Emacs.
 
 GNU Emacs is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
-the Free Software Foundation; either version 1, or (at your option)
+the Free Software Foundation; either version 2, or (at your option)
 any later version.
 
 GNU Emacs is distributed in the hope that it will be useful,
@@ -162,7 +162,8 @@ extern short ospeed;		/* Output speed (from sg_ospeed) */
 extern int cost;
 extern int evalcost ();
 
-extern void cmputc ();
+extern void cmcheckmagic ();
+extern int cmputc ();
 extern int cmcostinit ();
 extern int cmgoto ();
 extern int Wcm_clear ();
