@@ -31,10 +31,10 @@ the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.  */
 
 #define LONGBITS 32		/* Number of bits in a long */
 
-/* Define BIG_ENDIAN iff lowest-numbered byte in a word
+/* Define WORDS_BIG_ENDIAN iff lowest-numbered byte in a word
    is the most significant byte.  */
 
-#undef BIG_ENDIAN
+#undef WORDS_BIG_ENDIAN
 
 /* Now define a symbol for the cpu type, if your compiler
    does not define it automatically.  */
